# -*- coding: utf-8 -*-

'''Functions for performing network propagation
'''

# External library imports
import networkx as nx
import numpy as np
import pandas as pd
import warnings


def __init__(self):
    pass


def get_normalized_adjacency_matrix(graph, conserve_heat=True, weighted=False):
    """
    Returns normalized adjacency matrix (W'), as detailed in:

    Vanunu, Oron, et al. 'Associating genes and protein complexes with disease
    via network propagation.'

    :param graph: Interactome from which to calculate normalized
            adjacency matrix.
    :type graph: :py:class:`networkx.Graph`
    :param conserve_heat: If ``True``, heat will be conserved
            (ie. the sum of the heat vector will be equal to 1),
            and the graph will be asymmetric. Otherwise, heat will
            not be conserved, and the graph will be symmetric.
    :type conserve_heat: bool
    :param weighted: If ``True``, then the graph's edge weights
            will be taken into account. Otherwise, all edge weights
            will be set to 1.
    :type weighted: bool
    :return: Square normalized adjacency matrix
    :rtype: :py:class:`numpy.ndarray`
    """
    assert 0 not in dict(graph.degree).values(), "Graph cannot have nodes with degree=zero"

    # Create graph
    if conserve_heat:
        # If conserving heat, make G_weighted a di-graph (not symmetric)
        graph_weighted = nx.DiGraph()
    else:
        # If not conserving heat, make G_weighted a simple graph (symmetric)
        graph_weighted = nx.Graph()

    # Create edge weights
    edge_weights = []
    node_to_degree_dict = dict(graph.degree)
    if weighted and not nx.is_weighted(G=graph):
        warnings.warn("Input graph is not weighted. All edge weights will be set to 1.")

    for e in graph.edges(data=True):
        v1 = e[0]
        v2 = e[1]
        deg1 = node_to_degree_dict[v1]
        deg2 = node_to_degree_dict[v2]

        if weighted and nx.is_weighted(G=graph):
            weight = e[2]['weight']
        else:
            weight = 1

        if conserve_heat:
            # created asymmetrically weighted edges - each directed edge u->v normalized by the degree of v
            edge_weights.append((v1, v2, weight / float(deg1)))
            edge_weights.append((v2, v1, weight / float(deg2)))
        else:
            # normalize single undirected edge by the degree of both endpoints as per Vanunu, Oron, et al. 2010
            edge_weights.append((v1, v2, weight / np.sqrt(deg1 * deg2)))

    # Apply edge weights to graph
    graph_weighted.add_weighted_edges_from(edge_weights)

    # Transform graph to adjacency array
    if len(graph.nodes) != len(graph_weighted):
        raise ValueError("Input graph has nodes with zero degrees. Please remove these nodes.")

    w_prime = nx.to_numpy_array(graph_weighted, nodelist=graph.nodes())

    return w_prime


def get_individual_heats_matrix(normalized_adjacency_matrix, alpha=0.5):
    """
    Returns the pre-calculated contributions of each individual gene in the
    interactome to the final heat of each other gene in the interactome after
    propagation.

    :param normalized_adjacency_matrix: square normalized
            adjacency matrix from :py:func:`~netcoloc.netprop.get_normalized_adjacency_matrix`
    :type normalized_adjacency_matrix: :py:class:`numpy.ndarray`
    :param alpha: heat dissapation coefficient between 1 and 0. The
           contribution of the heat propagated from adjacent nodes in
           determining the final heat of a node, as opposed to the contribution
           from being a part of the gene set initially
    :type alpha: float
    :return: square individual heats matrix
    :rtype: :py:class:`numpy.ndarray`
    """
    assert 1 >= alpha >= 0, "Alpha must be between 0 and 1"
    # adjacency matrix must be transposed to allow compatability of asymmetric matrix with network propagation formula
    normalized_adjacency_matrix_transpose = np.transpose(normalized_adjacency_matrix)
    return np.linalg.inv(
        np.identity(normalized_adjacency_matrix_transpose.shape[0])
        - alpha * normalized_adjacency_matrix_transpose
    ) * (1 - alpha)


def network_propagation(individual_heats_matrix, nodes, seed_genes):
    """
    Implements network propagation, as detailed in:

    Vanunu, Oron, et al. 'Associating genes and protein complexes with
    disease via network propagation.'

    Using this function, the final heat of the network is calculated directly,
    instead of iteratively. This method is faster when many different
    propagations need to be performed on the same network (with different seed
    gene sets). It is slower than
    :py:func:`~netcoloc.netprop.iterative_network_propagation` for a
    single propagation.

    :param individual_heats_matrix: Square matrix that is the
                                    output of :py:func:`~netcoloc.netprop.get_individual_heats_matrix`
    :type individual_heats_matrix: :py:class:`numpy.ndarray`
    :param nodes: List of nodes in the network represented by the
            individual_heats_matrix, in the same order in which they were
            supplied to :py:func:`~netcoloc.netprop.get_individual_heats_matrix`
    :type nodes: list
    :param seed_genes: # TODO
    :return: Final heat of each node after propagation, with the name
             of the nodes as the index
    :rtype: :py:class:`pandas.Series`
    """
    # Remove genes that are not in network
    seed_genes = list(np.intersect1d(nodes, seed_genes))

    # Initialize results vector
    F = np.zeros(len(nodes))

    # Add up resulting heats from each gene in seed genes set
    for gene in seed_genes:
        # TODO check that this is the correct orientation
        F += individual_heats_matrix[:,nodes.index(gene)]

    # Normalize results by number of seed genes
    F /= len(seed_genes)

    #Return as pandas series
    # TODO does this need to be a pandas series?
    return pd.Series(F, index=nodes)



# -*- coding: utf-8 -*-

'''Functions for performing network localization
'''

# External library imports
import copy
import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns
from tqdm.auto import tqdm
import warnings

# Internal module convenience imports
from .netcoloc_utils import *
#from netcoloc_utils import *


def __init__(self):
    pass

# -------------------- LOCALIZATION ---------------------------------#


def netprop_localization(z_scores, random_final_heats,
                         seed_genes, z_score_threshold=3, plot=True):
    """
    Calculates the size of a gene set's network proximity based on the
    z-score results of network propagation, and evaluates if it is larger than
    chance by repeating the algorithm using random degree-matched seed genes.
    Returns a z-score.

    :param z_scores: The output of the
                     :py:func:`~netcoloc.netprop_zscore.netprop_zscore`
                     method. A pandas Series where the index
                     contains gene names, and the data contains
                     the z-scores associated with each gene after
                     network propagation.
    :type z_scores: :py:class:`pandas.Series`
    :param random_final_heats: The output of the
                               :py:func:`~netcoloc.netprop_zscore.netprop_zscore`
                               method. A matrix containing ``z_scores``,
                               where each column corresponds to a gene, and
                               each row corresponds to a network propagation
                               performed using random seed genes.
    :type random_final_heats: :py:class:`numpy.ndarray`
    :param seed_genes: list of seed genes used to generate the z_scores
            and random_final_heats arguments.
    :type seed_genes: list
    :param z_score_threshold: The threshold after which genes are
                              considered significantly proximal to each
                              other.
    :type z_score_threshold: float
    :param plot: If True, then the distribution will be plotted
    :type plot: bool
    :return: The z-score representing how much larger the network
             proximity of this seed gene is than would be expected
             by chance.
    :rtype: float
    """
    nan_row = [np.nan] * len(random_final_heats[0])

    random_proximal_network_sizes = []
    for row in tqdm(range(len(random_final_heats))):
        # Calculate mean and standard deviation, excluding focal row
        focal_row = copy.deepcopy(random_final_heats[row])
        random_final_heats[row] = nan_row
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            log = np.log(random_final_heats)
            mean = np.nanmean(log, axis=0)
            standard_deviation = np.nanstd(log, axis=0)
            # Calculate z-scores for each gene in random network
            random_z_score = (np.log(focal_row) - mean) / standard_deviation
        # Count gene in proximal network, not including seed genes
        random_proximal_network_sizes.append(sum((random_z_score > z_score_threshold)) - len(seed_genes))
        # Replace focal row in random final heats matrix
        random_final_heats[row] = copy.deepcopy(focal_row)

    # Calculate the size of the true proximal subgraph, not including seed genes
    proximal_network_size = sum((z_scores > z_score_threshold)) - len(seed_genes)

    # Calculate z-score
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        z_score = (np.log(proximal_network_size) - np.nanmean(np.log(random_proximal_network_sizes))) / np.nanstd(np.log(random_proximal_network_sizes))

    # Plot figure
    if plot:
        plt.figure(figsize=(5, 4))
        dfig = sns.histplot(random_proximal_network_sizes, label='Random')
        plt.vlines(proximal_network_size, ymin=0, ymax=dfig.dataLim.bounds[3], color='r', label='True gene set')
        plt.xlabel('Size of proximal network, z > ' + str(z_score_threshold), fontsize=16)
        plt.ylabel('Count', fontsize=16) 
        plt.legend(loc='upper left')

    return z_score, proximal_network_size, random_proximal_network_sizes, 



# -*- coding: utf-8 -*-

'''Functions for getting z-scores from network propagation.
'''

# External library imports

import os
import warnings
from tqdm.auto import tqdm
import ndex2
# Internal module convenience imports
from netcoloc.netcoloc_utils import *
from netcoloc.netprop import *


def __init__(self):
    pass


def netprop_zscore(seed_gene_file, seed_gene_file_delimiter=None, num_reps=10, alpha=0.5, minimum_bin_size=10,
                   interactome_file=None, interactome_uuid='f93f402c-86d4-11e7-a10d-0ac135e8bacf',
                   ndex_server='public.ndexbio.org', ndex_user=None, ndex_password=None, out_name='out',
                   save_z_scores=False, save_final_heat=False, save_random_final_heats=False, verbose=True):
    """
    Performs network heat propagation on the given interactome with the given
    seed genes, then returns the z-scores of the final heat values of each node
    in the interactome.

    The z-scores are calculated based on a null model, which is built by running
    the network propagation multiple times using randomly selected seed genes
    with similar degree distributions to the original seed gene set.

    This method returns a tuple containing the following:

    * :py:class:`pandas.Series` containing z-scores for each gene. Gene names comprise the index column
    * :py:class:`numpy.ndarray` containing square matrix where each row contains the final heat scores
      for each gene from a network propagation from random seed genes

    :param seed_gene_file: Location of file containing a delimited list of
            seed genes
    :type seed_gene_file: str
    :param seed_gene_file_delimiter: Delimiter used to separate genes in seed
                                     gene file. Default any whitespace
    :type seed_gene_file_delimiter: str
    :param num_reps: Number of times the network propagation algorithm should
            be run using random seed genes in order to build the null model
    :type num_reps: int
    :param alpha: Number between 0 and 1. Denotes the importance of the
            propagation step in the network propagation, as opposed to the step
            where heat is added to seed genes only. Recommended to be 0.5 or
            greater
    :type alpha: float
    :param minimum_bin_size: minimum number of genes that should be in
            each degree matching bin.
    :type minimum_bin_size: int
    :param interactome_file: Location of file containing the interactome in
            NetworkX gpickle format. Either the interactome_file argument or the
            interactome_uuid argument must be defined.
    :type interactome_file: str
    :param interactome_uuid: UUID of the interactome on NDEx. Either the
            interactome_file argument or the interactome_uuid argument must be
            defined. (Default: The UUID of PCNet, the Parsimonious Composite
            Network: f93f402c-86d4-11e7-a10d-0ac135e8bacf)
    :type interactome_uuid: str
    :param ndex_server: NDEx server on which the interactome is stored.
            Only needs to be defined if interactome_uuid is defined
    :type ndex_server: str
    :param ndex_user: NDEx user that the interactome belongs to. Only
            needs to be defined if interactome_uuid is defined, and the
            interactome is private
    :type ndex_user: str
    :param ndex_password: password of the NDEx user's account. Only needs
            to be defined if interactome_uuid is defined, and the interactome is
            private
    :type ndex_password: str
    :param out_name: Prefix for saving output files
    :type out_name: str
    :param save_z_scores:
    :param save_final_heat: If ``True``, then the raw network
            propagation heat scores for the original seed gene set will be saved
            in the form of a tsv file in the current directory
    :type save_final_heat: bool
    :param save_random_final_heats: If ``True``, then the raw
            network propagation heat scores for every repetition of the
            algorithm using random seed genes will be saved in the form of a tsv
            file in the current directory. (Beware: This can be a large file if
            num_reps is large.)
    :type save_random_final_heats: bool
    :param verbose: If ``True``, then progress information will
            be logged. Otherwise, nothing will be printed
    :return: (:py:class:`pandas.Series`, :py:class:`numpy.ndarray`)
    :rtype: tuple
    :raises TypeError: If neither interactome_file or interactome_uuid is provided or if
                       **num_reps** is not an ``int``
    """
    # Process arguments

    # seed_gene_file
    seed_gene_file = os.path.abspath(seed_gene_file)
    #num_reps
    try:
        num_reps = int(num_reps)
    except:
        raise TypeError("The num_reps argument should be an integer")
    #int_file and int_uuid
    if interactome_file is None and interactome_uuid is None:
        raise TypeError("Either interactome_file or interactome_uuid argument must be provided")

    # Load interactome
    if verbose:
        print('Loading interactome')
    if interactome_file is not None:
        interactome_file = os.path.abspath(interactome_file)
        interactome = nx.Graph()
        interactome = nx.read_gpickle(interactome_file)
    else:
        interactome = ndex2.create_nice_cx_from_server(
            ndex_server,
            username=ndex_user,
            password=ndex_password,
            uuid=interactome_uuid
        ).to_networkx()
    if 'None' in interactome.nodes():
        interactome.remove_node('None')
    nodes = list(interactome.nodes)

    # Log interactome num nodes and edges for diagnostic purposes
    if verbose:
        print('Number of nodes: ' + str(len(interactome.nodes)))
        print('Number of edges: ' + str(len(interactome.edges)))

    # Load seed genes
    seed_file = open(seed_gene_file, 'r')
    seed_genes = list(np.intersect1d(nodes, seed_file.read().split(seed_gene_file_delimiter)))
    if verbose:
        print('\nNumber of seed genes in interactome: ' + str(len(seed_genes)))

    # Calculate individual_heats_matrix from interactome
    if verbose:
        print('\nCalculating w_prime')
    w_prime = get_normalized_adjacency_matrix(interactome, conserve_heat=True)
    if verbose:
        print('\nCalculating individual_heats_matrix')
    individual_heats_matrix = get_individual_heats_matrix(w_prime, alpha)

    # Calculate the z-score
    if verbose:
        print('\nCalculating z-scores: ' + seed_gene_file)
    z_scores, final_heat, random_final_heats = calculate_heat_zscores(
        individual_heats_matrix,
        nodes,
        dict(interactome.degree),
        seed_genes,
        num_reps=num_reps,
        alpha=alpha,
        minimum_bin_size=minimum_bin_size)

    # Save z-score results
    z_scores.name = 'z-scores'
    if save_z_scores:
      z_scores.to_csv(out_name + '_z_scores_' + str(num_reps) + '_reps.tsv', sep='\t')

    # If save_final_heat is true, save out the final heat vector
    if save_final_heat:
        final_heat_df = pd.DataFrame(final_heat, columns=['z-scores'])
        final_heat_df.to_csv(out_name + '_final_heat_' + str(num_reps) + '_reps.tsv', sep='\t')

    # If save_random_final_heats is true, save out the vector of randoms (this can be a large file)
    if save_random_final_heats:
        random_final_heats_df = pd.DataFrame(
            random_final_heats.T,
            index=nodes,
            columns=range(1, random_final_heats.shape[0] + 1)
        )
        random_final_heats_df.to_csv(out_name + '_final_heat_random_' + str(num_reps) + '_reps.tsv', sep='\t')

    return z_scores, random_final_heats


def calculate_heat_zscores(individual_heats_matrix, nodes, degrees, seed_genes, num_reps=10, alpha=0.5, minimum_bin_size=10,random_seed=1):
    """
    Helper function to perform network heat propagation using the given
    individual heats matrix with the given seed genes and return the z-scores of
    the final heat values of each node.

    The z-scores are calculated based on a null model, which is built by running
    the network propagation multiple times using randomly selected seed genes
    with similar degree distributions to the original seed gene set.

    The returned tuple contains the following:

    * :py:class:`pandas.Series` containing z-scores for each gene. Gene names comprise the index column
    * :py:class:`pandas.Series` containing the final heat scores for each gene. Gene names comprise the index column,
    * :py:class:`numpy.ndarray` containing square matrix in which each row contains the final heat scores for each gene from a network propagation from random seed genes)

    :param individual_heats_matrix: output of the
            netprop.get_individual_heats_matrix. A square matrix containing the
            final heat contributions of each gene
    :type individual_heats_matrix: :py:class:`numpy.ndarray`
    :param nodes: nodes, in the order in which they were supplied to
            the :py:func:`~netcoloc.netprop.get_normalized_adjacency_matrix` method
            which returns the precursor to the individual_heats_matrix
    :type nodes: list
    :param degrees: Mapping of node names to node degrees
    :type degrees: dict
    :param seed_genes: list of genes to use for network propagation. The
            results of this network propagation will be compared to a set of
            random results in order to obtain z-scores
    :type seed_genes: list
    :param num_reps: Number of times the network propagation algorithm should
            be run using random seed genes in order to build the null model
    :type num_reps: int
    :param alpha: Number between 0 and 1. Denotes the importance of the
            propagation step in the network propagation, as opposed to the step
            where heat is added to seed genes only. Recommended to be 0.5 or
            greater
    :type alpha: float
    :param minimum_bin_size: minimum number of genes that should be in
            each degree matching bin
    :type minimum_bin_size: int
    :param random_seed:
    :return: (:py:class:`pandas.Series`, :py:class:`pandas.Series`, :py:class:`numpy.ndarray`)
    :rtype: tuple
    """
    # set random seed for reproducibility
    np.random.seed(random_seed)

    # Calculate network propagation results given gene set
    seed_genes = list(np.intersect1d(nodes, seed_genes))
    final_heat = network_propagation(individual_heats_matrix, nodes, seed_genes)

    # Initialize empty matrix for results of random network propagations
    random_final_heats = np.zeros([num_reps, len(final_heat)])

    # Create bins containing genes of similar degree
    bins, actual_degree_to_bin_index = get_degree_binning(degrees, minimum_bin_size)

    # Perform network propagation many times with random seed genes
    for repetition in tqdm(range(num_reps)):
        # Create list of random, degree-matched seed genes
        random_seed_genes = []
        for gene in seed_genes:
            # Find genes with similar degrees to focal gene degree
            degree = degrees[gene]
            genes_of_similar_degree = bins[actual_degree_to_bin_index[degree]]
            # Shuffle the genes in the bin
            np.random.shuffle(genes_of_similar_degree)

            # Add genes to list that haven't already been added
            index = 0
            while genes_of_similar_degree[index] in random_seed_genes:
                index += 1
            random_seed_genes.append(genes_of_similar_degree[index])

        # Perform network propagation with random seed genes
        random_final_heat = network_propagation(individual_heats_matrix, nodes, random_seed_genes)
        # Set seeds to NaN so they don't bias results
        random_final_heat.loc[random_seed_genes] = np.nan
        # Add results to random_final_heats matrix
        random_final_heats[repetition] = random_final_heat

    # Calculate z-scores
    with warnings.catch_warnings():
      warnings.simplefilter("ignore")
      z_scores = (np.log(final_heat) - np.nanmean(np.log(random_final_heats), axis=0)) / np.nanstd(np.log(random_final_heats), axis=0)

    return z_scores, final_heat, random_final_heats



# -*- coding: utf-8 -*-

'''Utility functions useful across multiple modules.
'''


def get_degree_binning(node_to_degree_dict, min_bin_size, lengths=None):
    """
    Groups nodes by degree into similarly sized bins. This function
    comes from
    `network_utilities.py of emreg00/toolbox <https://github.com/emreg00/toolbox/blob/master/network_utilities.py>`__


    Returns a tuple with following two values:

    * **list of bins** where each bin contains a list of nodes of similar degree
    * **mapping of degree to index of bin** dict mapping a degree to the index
      of the bin in the bins list which contains nodes of that degree

    :param node_to_degree_dict: Map of nodes to their degrees
    :type node_to_degree_dict: dict
    :param min_bin_size: minimum number of nodes each bin should contain.
    :type min_bin_size: int
    :param lengths: List of nodes to bin. If lengths is equal to None, then
                    all nodes will be binned
    :type lengths: list
    :return: (list of bins, mapping of degree to index of bin)
    :rtype: tuple
    """
    # Create dictionary mapping degrees to nodes
    degree_to_nodes = {}
    for node, degree in node_to_degree_dict.items():
        if lengths is not None and node not in lengths:
            continue
        degree_to_nodes.setdefault(degree, []).append(node)
    
    # Get sorted list of degrees
    degrees = degree_to_nodes.keys()
    degrees = list(degrees)
    degrees.sort()
    
    bins = []
    bins_boundaries = []
    degree_to_bin_index = {}

    degree_index = 0
    while degree_index < len(degrees):
        # Add nodes of each degree to bin until bin reaches minimum bin size
        low = degrees[degree_index]
        nodes_of_certain_degree = degree_to_nodes[low]
        while len(nodes_of_certain_degree) < min_bin_size:
            degree_index += 1
            if degree_index == len(degrees):
                degree_index -= 1
                break
            nodes_of_certain_degree.extend(degree_to_nodes[degrees[degree_index]])

        high = degrees[degree_index]
        if len(nodes_of_certain_degree) >= min_bin_size:
            # For each degree represented in bin, set degree to bin index
            for deg in range(low, high + 1):
                degree_to_bin_index[deg] = len(bins)
            bins.append(nodes_of_certain_degree)
            bins_boundaries.append((low, high))
        else:
            # Combine last bin with second last bin, if last bin is too small
            bins[-1].extend(nodes_of_certain_degree)
            low_of_previous_bin, high_of_previous_bin = bins_boundaries[-1]
            bins_boundaries[-1] = (low_of_previous_bin, high)
            for deg in range(high_of_previous_bin + 1, high + 1):
                degree_to_bin_index[deg] = len(bins) - 1
            
        degree_index += 1 

    return bins, degree_to_bin_index



# -*- coding: utf-8 -*-

'''Functions for performing validation of NetColoc subgraph
'''

import warnings
import numpy as np
import pandas as pd

import requests
from scipy.stats import hypergeom
from statsmodels.stats import contingency_tables

try:
    # need ddot to parse the ontology
    import ddot
    from ddot import Ontology
    DDOT_LOADED = True
except ImportError as ie:
    warnings.warn('Import of ddot failed. NetColoc will not work properly: + ' + str(ie))
    DDOT_LOADED = False

# find human orthologs of mouse genes
import mygene
mg = mygene.MyGeneInfo()


def load_MGI_mouseKO_data(url='http://www.informatics.jax.org/downloads/reports/MGI_PhenoGenoMP.rpt'):
    """
    Function to parse and load mouse knockout data from MGI.

    :param url: location of MGI knockout data
    :type url: str
    :return: parsed MGI knockout dataframe, including column for human orthologs
    :rtype: :py:class:`pandas.DataFrame`
    """
    # download MGI phenotype data
    r = requests.get(url,allow_redirects=True)
    open('MGI_PhenoGenoMP.rpt', 'wb').write(r.content)

    # parse the downloaded MGI phenotype data
    mgi_df = pd.read_csv('MGI_PhenoGenoMP.rpt', sep='\t',
                        names=['MGI_Allele_Accession_ID',
                               'Allele symbol', 'involves',
                               'MP', 'PMID', 'MGI_marker_accession_ID'])
    # extract gene names
    gene_name = [a.split('<')[0] for a in mgi_df['Allele symbol'].tolist()]
    mgi_df['gene_name']=gene_name
    mgi_df.index=mgi_df['gene_name']

    # map mouse genes to human orthologs
    mouse_genes = list(np.unique(mgi_df['gene_name']))
    mg_mapped = mg.querymany(mouse_genes,
                             as_dataframe=True, species=['mouse','human'],
                             scopes='symbol', fields='symbol')

    # drop genes with no human ortholog
    print(len(mg_mapped))
    mg_mapped = mg_mapped.dropna(subset=['symbol'])
    print(len(mg_mapped))
    # drop duplicates
    mg_mapped = mg_mapped[~mg_mapped.index.duplicated(keep='first')]
    print(len(mg_mapped))
    mg_mapped.head()

    mgi_df['human_ortholog']=mgi_df['gene_name'].map(dict(mg_mapped['symbol']))

    return mgi_df


def load_MPO(url='http://www.informatics.jax.org/downloads/reports/MPheno_OBO.ontology'):
    """
    Function to parse and load mouse phenotype ontology, using DDOT's ontology module

    :param url: URL containing MPO ontology file
    :type url: str
    :return: MPO parsed using DDOT
    :rtype: :py:class:`ddot.Ontology`
    :raises ImportError: If DDOT package is not found
    """

    # download the mammalian phenotype ontology, parse with ddot
    r = requests.get(url,allow_redirects=True)
    open('MPheno_OBO.ontology','wb').write(r.content)
    if DDOT_LOADED is False:
        raise ImportError('ddot package is required to use this method')
    ddot.parse_obo('MPheno_OBO.ontology',
                   'parsed_mp.txt',
                  'id2name_mp.txt',
                  'id2namespace_mp.txt',
                  'altID_mp.txt')


    MP2desc = pd.read_csv('id2name_mp.txt',sep='\t',
                          names=['MP','description'],index_col='MP')

    MP2desc=MP2desc.loc[MP2desc.index.dropna()] # drop NAN from index
    print(len(MP2desc))


    display(MP2desc.head())

    hierarchy = pd.read_table('parsed_mp.txt',
                              sep='\t',
                              header=None,
                              names=['Parent', 'Child', 'Relation', 'Namespace'])

    display(hierarchy.head())

    MPO = Ontology.from_table(
        table=hierarchy,
        parent='Parent',
        child='Child',
        add_root_name='MP:00SUPER',
        ignore_orphan_terms=True)

    # add description to node attribute
    terms_keep = list(np.unique(hierarchy['Parent'].tolist()+hierarchy['Child'].tolist()))
    MPO.node_attr=MP2desc.loc[terms_keep]

    return MPO


def MPO_enrichment_root(hier_df,MPO,mgi_df,MP_focal_list,G_int,verbose=True):
    """
    Function to test for enrichment of genes resulting in selected phenotypes
    when knocked out in root node of NetColoc hierarchy.

    The returned :py:class:`pandas.DataFrame` will have the following columns:

    * **OR_p** - Odds ratio p-vlaue
    * **log_OR** - natural log odds ratio
    * **log_OR_CI_lower** - lower 95% confidence interval on log_OR
    * **log_OR_CI_upper** - upper 95% confidence interval on log_OR
    * **num_genes_in_term** - number of genes in MPO term
    * **MP_description** - description of MPO phenotype

    :param hier_df: NetColoc systems map (processed output from cdaps_util)
    :type hier_df: :py:class:`pandas.DataFrame`
    :param MPO: DDOT ontology containing the parsed mammalian phenotype ontology
    :type MPO: :py:class:`ddot.Ontology`
    :param mgi_df: parsed MGI knockout dataframe
    :type mgi_df: :py:class:`pandas.DataFrame`
    :param MP_focal_list: List of MPO phenotypes to check for enrichment against
    :type MP_focal_list: list
    :param G_int: Background interactome
    :type G_int: :py:class:`networkx.Graph`
    :param verbose: If true, print out some progress
    :type verbose: bool
    :return: Dataframe containing enrichment results
    :rtype: :py:class:`pandas.DataFrame`
    """
    # test for enrichment in root node
    OR_p_list, OR_CI_list,log_OR_list = [], [], []
    num_genes_in_term_list = []

    MP_keep_list = []

    # root node is the largest node
    hier_df.index=hier_df['name']
    root_node = hier_df['CD_MemberList_Size'].sort_values(ascending=False).head(1).index.tolist()[0]

    # get list of node names from G_int
    G_int_nodes = list(G_int.nodes())

    for MP_focal in MP_focal_list:
        MP_desc_focal = dict(MPO.node_attr['description'])[MP_focal]

        # focus the hierarchy on one branch, and look up all terms within that branch
        if len(MPO.parent_2_child[MP_focal])>0:
            MPO_focal = MPO.focus(MP_focal,verbose=False)
            focal_terms = MPO_focal.terms
        else:  # if the term has no children, just look at that term
            focal_terms=[MP_focal]

        # check enrichment in root node
        focal_genes = hier_df['CD_MemberList'].loc[root_node].split(' ')
        mgi_temp = mgi_df[mgi_df['MP'].isin(focal_terms)]
        mgi_temp = mgi_temp.dropna(subset=['human_ortholog'])
        mgi_genes = list(np.unique(mgi_temp['human_ortholog']))
        mgi_genes = list(np.intersect1d(mgi_genes,G_int_nodes))

        # only test if there are at least 10 genes, and fewer than 2000 genes
        if (len(mgi_genes)>10) and (len(mgi_genes) < 2000):
            q00 = len(np.intersect1d(mgi_genes, focal_genes))

            q01 = len(mgi_genes)-q00
            q10 = len(focal_genes)-q00
            q11 = len(G_int_nodes)-q00-q01-q10
            table_temp = [[q00,q01],[q10,q11]]

            CT= contingency_tables.Table2x2(table_temp)
            OR_p_temp = CT.log_oddsratio_pvalue()
            OR_CI_temp = CT.log_oddsratio_confint()
            log_OR_temp = CT.log_oddsratio

            OR_p_list.append(OR_p_temp)
            OR_CI_list.append(OR_CI_temp)
            log_OR_list.append(log_OR_temp)
            num_genes_in_term_list.append(len(mgi_genes))

            MP_keep_list.append(MP_focal)

            if verbose:
                print('\n'+MP_desc_focal)
                print('number of genes in root node = '+str(len(focal_genes)))
                print('number of genes in focal MPO term = '+str(len(mgi_genes)))
                print('number overlapping genes = '+str(q00))
                print(OR_p_temp)
                print(OR_CI_temp)
                print(log_OR_temp)

    OR_CI_lower, OR_CI_upper = zip(*OR_CI_list)

    root_KO_df = pd.DataFrame({'OR_p':OR_p_list,'log_OR':log_OR_list,
                               'log_OR_CI_lower':OR_CI_lower,'log_OR_CI_upper':OR_CI_upper,
                              'num_genes_in_term':num_genes_in_term_list},
                              index=MP_keep_list)

    root_KO_df['MP_description'] = root_KO_df.index.map(dict(MPO.node_attr['description']))

    return root_KO_df


def MPO_enrichment_full(hier_df,MPO,mgi_df,MP_focal_list,G_int):
    """
    Function to test for enrichment of genes resulting in selected phenotypes
    when knocked out in every NetColoc system (not just root)

    The returned :py:class:`pandas.DataFrame` will have these columns:

    * **log(OR_p)** - -log10(Odds ratio p-vlaue)
    * **log_OR** - natural log odds ratio
    * **num_genes** - number of genes in MPO term overlapping with focal system
    * **gene_ids** - list of overlapping genes between MPO term and focal system

    :param hier_df: NetColoc systems map (processed output from cdaps_util)
    :type hier_df: :py:class:`pandas.DataFrame`
    :param MPO: DDOT ontology containing the parsed mammalian phenotype ontology
    :type MPO: :py:class:`ddot.Ontology`
    :param mgi_df: parsed MGI knockout dataframe
    :type mgi_df: :py:class:`pandas.DataFrame`
    :param MP_focal_list: List of MPO phenotypes to check for enrichment against
    :type MP_focal_list: list
    :param G_int: Background interactome
    :type G_int: :py:class:`networkx.Graph`
    :return: Dataframe containing enrichment results
    :rtype: :py:class:`pandas.DataFrame`
    """
    MP_full_results_df=pd.DataFrame(index=hier_df.index.tolist())

    for MP_focal in MP_focal_list:
        MP_desc_focal = dict(MPO.node_attr['description'])[MP_focal]
        print(MP_desc_focal)

        # focus the hierarchy on one branch, and look up all terms within that branch
        if len(MPO.parent_2_child[MP_focal])>0:
            MPO_focal = MPO.focus(MP_focal)
            focal_terms = MPO_focal.terms
        else:  # if the term has no children, just look at that term
            focal_terms=[MP_focal]

        hyper_p_list = []
        num_genes_list = []
        genes_id_list = []

        OR_p_list,OR_CI_list,log_OR_list=[],[],[]
        for focal_cluster in hier_df.index.tolist():
            mFocal_genes = hier_df['CD_MemberList'].loc[focal_cluster].split(' ')

            M=len(list(G_int.nodes())) # only keep genes in PCnet
            # Look up all entries matching focal_terms, and mFocal_genes
            mgi_temp = mgi_df[mgi_df['MP'].isin(focal_terms)]
            mgi_temp = mgi_temp.dropna(subset=['human_ortholog'])
            mgi_genes = list(np.unique(mgi_temp['human_ortholog']))
            new_index=[g.upper() for g in mgi_temp.index.tolist()]
            mgi_temp.index=new_index

            # only keep genes in PCnet
            N=len(np.intersect1d(list(np.unique(mgi_temp.index.tolist())),
                                 list(G_int.nodes())))

            if len(np.intersect1d(mFocal_genes,mgi_temp.index.tolist()))>0:
                mgi_genes = mgi_temp.index.tolist()
                mgi_genes = list(np.intersect1d(mgi_genes,list(G_int.nodes())))
                x = len(np.intersect1d(mFocal_genes,mgi_genes))
                n=len(mFocal_genes)

                hyper_p_list.append(hypergeom.sf(x,M,n,N))
                num_genes_list.append(x)
                genes_id_list.append(' '.join(list(np.intersect1d(mFocal_genes,mgi_genes))))

                q00 = len(np.intersect1d(mgi_genes,mFocal_genes))
                q01 = len(mgi_genes)-q00

                q10 = len(mFocal_genes)-q00
                q11 = len(list(G_int.nodes()))-q00-q01-q10

                table_temp = [[q00,q01],[q10,q11]]

                CT= contingency_tables.Table2x2(table_temp)
                OR_p_temp = CT.log_oddsratio_pvalue()
                OR_CI_temp = CT.log_oddsratio_confint()
                log_OR_temp = CT.log_oddsratio

                OR_p_list.append(OR_p_temp)
                OR_CI_list.append(OR_CI_temp)
                log_OR_list.append(log_OR_temp)

            else:
                hyper_p_list.append(1)
                num_genes_list.append(0)
                genes_id_list.append('')

                OR_p_list.append(1)
                OR_CI_list.append(0)
                log_OR_list.append(0)

        MP_focal_df = pd.DataFrame({MP_desc_focal+':-log(OR_p)':-np.log10(OR_p_list),
                                    MP_desc_focal+':log_OR':log_OR_list,
                                    MP_desc_focal+':num_genes':num_genes_list,
                                    MP_desc_focal+':gene_ids':genes_id_list},index=hier_df.index.tolist())

        # check that the column isn't already present in MP_full_results_df
        if MP_desc_focal+':-log(OR_p)' not in MP_full_results_df.columns.tolist():
            MP_full_results_df=MP_full_results_df.join(MP_focal_df)

    return MP_full_results_df



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_netcoloc
----------------------------------

Tests for `netcoloc` module.
"""


import sys
import unittest
from contextlib import contextmanager
from click.testing import CliRunner

#from netcoloc import netcoloc
from netcoloc import cli
from netcoloc import netprop
from netcoloc import netprop_zscore
from netcoloc import network_localization
from netcoloc import network_colocalization
from netcoloc import netcoloc_utils

class TestNetcoloc(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_000_something(self):
        pass

    #def test_command_line_interface(self):
     #   runner = CliRunner()
      #  result = runner.invoke(cli.main)
       # assert result.exit_code == 0
        #assert 'netcoloc.cli.main' in result.output
        #help_result = runner.invoke(cli.main, ['--help'])
        #assert help_result.exit_code == 0
        #assert '--help  Show this message and exit.' in help_result.output


if __name__ == '__main__':
    sys.exit(unittest.main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_netcoloc
----------------------------------

Tests for `netcoloc` module.
"""


import sys
import unittest
import os
import ndex2
import networkx as nx
import numpy as np

from netcoloc import netprop


class TestNetcoloc(unittest.TestCase):

    def setUp(self):
        """
        Create directed and undirected toy example networks.
        :return:
        """
        net = ndex2.create_nice_cx_from_file(self._get_5node_network())
        self.g = net.to_networkx(mode='default')
        graph_undirected = nx.Graph()
        graph_undirected.add_nodes_from([1, 2, 3])
        graph_undirected.add_edges_from([(1, 2), (2, 3)])
        self.undirected = graph_undirected
        graph_directed = nx.DiGraph()
        graph_directed.add_nodes_from([1, 2, 3])
        graph_directed.add_weighted_edges_from([(1, 2, 1), (2, 1, 0.5), (2, 3, 0.5), (3, 2, 1)])
        self.directed = graph_directed


    def tearDown(self):
        pass

    def _get_5node_network(self):
        """
        Gets testppi.cx from data directory
        under tests
        :return:
        """
        testdir = os.path.dirname(__file__)
        return os.path.join(testdir, 'data',
                            '5node.cx')

    def _get_testppi_network(self):
        """
        Gets testppi.cx from data directory
        under tests
        :return:
        """
        testdir = os.path.dirname(__file__)
        return os.path.join(testdir, 'data',
                            'testppi.cx')

    def test_000_something(self):
        assert 'one' == 'one'

    def test_netprop_undirected(self):
        """
        Test that method returns correct result for undirected graph i.e. symmetrical adjacency matrix
        :return:
        """
        adj = nx.to_numpy_array(self.undirected)
        heats = netprop.get_individual_heats_matrix(adj, alpha=0.5)
        F = netprop.network_propagation(heats, list(self.undirected.nodes), [1])
        expected = [0.75, 0.5, 0.25]
        for i, node in enumerate(F.keys()):
            self.assertAlmostEqual(expected[i], F[node])

    def test_netprop_directed_unidirectional(self):
        """
        Test that model can handle directed networks having nodes with uneven in/out degree
        :return:
        """
        adj = nx.to_numpy_array(self.g)
        heats = netprop.get_individual_heats_matrix(adj, alpha=0.5)
        F1 = netprop.network_propagation(heats, list(self.g.nodes), [185])
        # 1 - test no propagation out of seed with out-degree=0
        expected1 = [0, 0, 0.5, 0, 0]
        for i, node in enumerate(F1.keys()):
            self.assertAlmostEqual(expected1[i], F1[node])
        # 2 - test propagation out of seeds with out-degree > 0
        F2 = netprop.network_propagation(heats, list(self.g.nodes), [180, 190])
        expected2 = [0.25, 0.25, 0.125, 0.25, 0.25]
        for i, node in enumerate(F2.keys()):
            self.assertAlmostEqual(expected2[i], F2[node])

    def test_netprop_directed_bidirectional(self):
        """
        Test that method returns correct result for directed network with all nodes have equal in/out degree,
        but asymmetric adjacency matrix.
        :return:
        """
        adj = nx.to_numpy_array(self.directed)
        heats = netprop.get_individual_heats_matrix(adj, alpha=0.5)
        F = netprop.network_propagation(heats, list(self.directed.nodes), [1])
        expected = [7/12, 2/6, 1/12]
        for i, node in enumerate(F.keys()):
            self.assertAlmostEqual(expected[i], F[node], places=5)

    def test_netprop_unconnected(self):
        """
        Test that method appropriately deals with networks that contain unconnected components
        :return:
        """
        g32 = self.g.copy()
        g32.remove_edge(180, 195, 0)
        g32.remove_edge(190, 175, 0)
        # network now has two components (195, 190) and (185, 180, 175)
        adj = nx.to_numpy_array(g32)
        heats = netprop.get_individual_heats_matrix(adj, alpha=0.5)
        F = netprop.network_propagation(heats, list(g32.nodes), [180])
        # test that nodes with no path to seed get no heat
        self.assertEqual(0, F[195])
        self.assertEqual(0, F[190])
        # test that nodes with path to seed do get heat
        for node in [185, 180, 175]:
            self.assertGreater(F[node], 0)

    def test_netprop_alpha(self):
        """
        Test that the resulting heats change appropriately when alpha is modified from default.
        :return:
        """
        adj = nx.to_numpy_array(self.undirected)
        heats = netprop.get_individual_heats_matrix(adj, alpha=0.1)
        F = netprop.network_propagation(heats, list(self.undirected.nodes), [1])
        expected = [891/980, 9/98, 9/980]
        for i, node in enumerate(F.keys()):
            self.assertAlmostEqual(expected[i], F[node])

    def test_normalization_multigraph(self):
        #TODO
        pass

    def test_normalization_undirected(self):
        #TODO
        pass

    def test_normalization_directed(self):
        #TODO
        pass

    def test_normalization_conservation(self):
        #TODO
        pass

    def test_normalization_weighting(self):
        #TODO
        pass

    def test_normalization_warns_weighted_unweighted_input(self):
        #TODO
        pass

    def test_normalization_warns_degree_zero(self):
        g41 = self.g.copy()
        g41.remove_edge(180, 195, 0)
        g41.remove_edge(190, 195, 0)
        self.assertRaises(AssertionError, netprop.get_normalized_adjacency_matrix, g41)

    def test_heats_symmetric(self):
        #TODO
        pass

    def test_heats_asymmetric(self):
        #TODO
        pass

    def test_heats_catch_bad_alpha(self):
        self.assertRaises(AssertionError, netprop.get_individual_heats_matrix, np.array([0]), alpha=10)
        self.assertRaises(AssertionError, netprop.get_individual_heats_matrix, np.array([0]), alpha=-1)


if __name__ == '__main__':
    sys.exit(unittest.main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_netcoloc
----------------------------------

Tests for `netcoloc` module.
"""


import os
import sys
import unittest
import json
import ndex2
import networkx as nx
import pandas as pd

from netcoloc import network_colocalization


class TestNetworkColocalization(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def _get_5node_network(self):
        """
        Gets testppi.cx from data directory
        under tests
        :return:
        """
        testdir = os.path.dirname(__file__)
        return os.path.join(testdir, 'data',
                            '5node.cx')

    def _get_testppi_network(self):
        """
        Gets testppi.cx from data directory
        under tests
        :return: 
        """
        testdir = os.path.dirname(__file__)
        return os.path.join(testdir, 'data',
                            'testppi.cx')

    def test_transform_edges_5node_network(self):
        net = ndex2.create_nice_cx_from_file(self._get_5node_network())

        g = net.to_networkx(mode='default')

        name_dict = {}
        for entry in g.nodes.data():
            name_dict[entry[1]['name']] = entry[0]

        res = network_colocalization.transform_edges(g)

        self.assertEqual(2, len(list(res.edges())))
        self.assertEqual(5, len(list(res.nodes())))
        self.assertEqual(1.125, res.get_edge_data(name_dict['B'],
                                                  name_dict['D'])['weight'])
        self.assertEqual(1.0, res.get_edge_data(name_dict['A'],
                                                name_dict['E'])['weight'])

    @unittest.skip('Tests overlap network from notebook. Too slow...')
    def test_transform_edges_testppi_network(self):
        net = ndex2.create_nice_cx_from_file(self._get_testppi_network())

        g = net.to_networkx(mode='default')

        name_dict = {}
        for entry in g.nodes.data():
            name_dict[entry[1]['name']] = entry[0]

        res = network_colocalization.transform_edges(g)
        self.assertEqual(7862, len(list(res.edges())))
        self.assertEqual(773, len(list(res.nodes())))


if __name__ == '__main__':
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-

'''Functions for performing network colocalization
'''


import warnings
import logging

# External library imports
import matplotlib.pyplot as plt
import networkx as nx
import numpy as np
import pandas as pd
import seaborn as sns
from scipy.spatial import distance


def __init__(self):
    pass


logger = logging.getLogger(__name__)


def calculate_network_overlap(z_scores_1, z_scores_2, z_score_threshold=3,
                              z1_threshold=1.5, z2_threshold=1.5):
    """
    Function to determine which genes overlap. Returns a list of the
    overlapping genes

    :param z_scores_1: Result from :py:func:`~netcoloc.netprop_zscore.netprop_zscore`
                       or :py:func:`~netcoloc.netprop_zscore.calculate_heat_zscores`
                       containing the z-scores of each gene following network
                       propagation. The index consists of gene names
    :type z_scores_1: :py:class:`pandas.Series`
    :param z_scores_2: Similar to **z_scores_1**. This and **z_scores_1**
                       must contain the same genes (ie. come from the same
                       interactome network)
    :type z_scores_2: :py:class:`pandas.Series`
    :param z_score_threshold: threshold to determine whether a gene is
        a part of the network overlap or not. Genes with combined z-scores
        below this threshold will be discarded
    :type z_score_threshold: float
    :param z1_threshold: individual z1-score threshold to determine whether a gene is
        a part of the network overlap or not. Genes with z1-scores
        below this threshold will be discarded
    :type z1_threshold: float
    :param z2_threshold: individual z2-score threshold to determine whether a gene is
        a part of the network overlap or not. Genes with z2-scores
        below this threshold will be discarded
    :type z2_threshold: float
    :return: genes in the network overlap (genes with high combined
            z-scores)
    :rtype: list
    """
    z_scores_1 = z_scores_1.to_frame(name='z_scores_1')
    z_scores_2 = z_scores_2.to_frame(name='z_scores_2')
    z_scores_joined = z_scores_1.join(z_scores_2)
    z_scores_combined = (z_scores_joined['z_scores_1']
                        * z_scores_joined['z_scores_2']
                        * (z_scores_joined['z_scores_1'] > 0)
                        * (z_scores_joined['z_scores_2'] > 0))
    # get rid of unlikely genes which have low scores in either z1 or z2
    high_z_score_genes = z_scores_combined[
        (z_scores_combined >= z_score_threshold)
         & (z_scores_joined['z_scores_1'] > z1_threshold)
         & (z_scores_joined['z_scores_2'] > z2_threshold)
    ].index.tolist()

    return high_z_score_genes


def calculate_network_overlap_subgraph(interactome, z_scores_1,
                                       z_scores_2, z_score_threshold=3,
                                       z1_threshold=1.5,z2_threshold=1.5):
    """
    Function to return subgraph of network intersection.

    Code to create subgraph is from
    `NetworkX subgraph documentation <https://networkx.org/documentation/stable/reference/classes/generated/networkx.Graph.subgraph.html>`__

    :param interactome: network whose subgraph will be returned
    :type interactome: :py:class:`networkx.Graph`
    :param z_scores_1: Result from :py:func:`~netcoloc.netprop_zscore.netprop_zscore`
                       or :py:func:`~netcoloc.netprop_zscore.calculate_heat_zscores`
                       containing the z-scores of each gene following network
                       propagation. The index consists of gene names
    :type z_scores_1: :py:class:`pandas.Series`
    :param z_scores_2: Similar to **z_scores_1**. This and **z_scores_1**
                       must contain the same genes (ie. come from the same
                       interactome network)
    :type z_scores_2: :py:class:`pandas.Series`
    :param z_score_threshold: threshold to determine whether a gene is
        a part of the network overlap or not. Genes with combined z-scores
        below this threshold will be discarded
    :type z_score_threshold: float
    :param z1_threshold: individual z1-score threshold to determine whether a gene is
        a part of the network overlap or not. Genes with z1-scores
        below this threshold will be discarded
    :type z1_threshold: float
    :param z2_threshold: individual z2-score threshold to determine whether a gene is
        a part of the network overlap or not. Genes with z2-scores
        below this threshold will be discarded
    :type z2_threshold: float
    :return: Subgraph of the interactome containing only genes that
            are in the network intersection (genes with high combined z-scores)
    :rtype: :py:class:`networkx.Graph`
    """
    network_overlap = calculate_network_overlap(z_scores_1, z_scores_2, z_score_threshold=z_score_threshold,
                                               z1_threshold=z1_threshold,z2_threshold=z2_threshold)

    # Create subgraph that has the same type as original graph
    network_overlap_subgraph = interactome.__class__()
    network_overlap_subgraph.add_nodes_from((node, interactome.nodes[node]) for node in network_overlap)
    if network_overlap_subgraph.is_multigraph():
        network_overlap_subgraph.add_edges_from((node, neighbor, key, dictionary)
            for node, neighbors in interactome.adj.items() if node in network_overlap
            for neighbor, item in neighbors.items() if neighbor in network_overlap
            for key, dictionary in item.items())
    else:
        network_overlap_subgraph.add_edges_from((node, neighbor, dictionary)
            for node, neighbors in interactome.adj.items() if node in network_overlap
            for neighbor, dictionary in neighbors.items() if neighbor in network_overlap)
    network_overlap_subgraph.graph.update(interactome.graph)

    return network_overlap_subgraph


def calculate_expected_overlap(z_scores_1, z_scores_2,
                               z_score_threshold=3, z1_threshold=1.5,
                               z2_threshold=1.5,
                               num_reps=1000, plot=False):
    """
    Determines size of expected network overlap by randomly
    shuffling gene names

    :param z_scores_1: Result from :py:func:`~netcoloc.netprop_zscore.netprop_zscore`
                       or :py:func:`~netcoloc.netprop_zscore.calculate_heat_zscores`
                       containing the z-scores of each gene following network
                       propagation. The index consists of gene names
    :type z_scores_1: :py:class:`pandas.Series`
    :param z_scores_2: Similar to **z_scores_1**. This and **z_scores_1**
                       must contain the same genes (ie. come from the same
                       interactome network)
    :type z_scores_2: :py:class:`pandas.Series`
    :param z_score_threshold: threshold to determine whether a gene is
        a part of the network overlap or not. Genes with combined z-scores
        below this threshold will be discarded
    :type z_score_threshold: float
    :param z1_threshold: individual z1-score threshold to determine whether a gene is
        a part of the network overlap or not. Genes with z1-scores
        below this threshold will be discarded
    :type z1_threshold: float
    :param z2_threshold: individual z2-score threshold to determine whether a gene is
        a part of the network overlap or not. Genes with z2-scores
        below this threshold will be discarded
    :type z2_threshold: float
    :param num_reps:
    :param plot: If ``True``, distribution will be plotted
    :type plot: bool
    :return:
    :rtype: float
    """
    # Build a distribution of expected network overlap sizes by shuffling node names
    random_network_overlap_sizes = []
    z_scores_1_copy = z_scores_1.copy()
    z_scores_2_copy = z_scores_2.copy()
    gene_set_1 = z_scores_1.index.tolist()
    gene_set_2 = z_scores_2.index.tolist()
    for _ in range(num_reps):
        # Shuffle gene name labels
        np.random.shuffle(gene_set_1)
        z_scores_1_copy.index = gene_set_1

        np.random.shuffle(gene_set_2)
        z_scores_2_copy.index = gene_set_2

        random_size = len(calculate_network_overlap(z_scores_1_copy, z_scores_2_copy,
                                                    z_score_threshold=z_score_threshold,
                                                    z1_threshold=z1_threshold,
                                                    z2_threshold=z2_threshold))
        random_network_overlap_sizes.append(random_size)

    network_overlap_size = len(calculate_network_overlap(z_scores_1, z_scores_2,
                                                         z_score_threshold=z_score_threshold,
                                                         z1_threshold=z1_threshold,
                                                         z2_threshold=z2_threshold))

    if plot:
        plt.figure(figsize=(5, 4))
        dfig = sns.histplot(random_network_overlap_sizes,
                            label='Expected network intersection size')
        plt.vlines(network_overlap_size, ymin=0, ymax=dfig.dataLim.bounds[3], color='r',
                   label='Observed network intersection size')
        plt.xlabel('Size of proximal subgraph, z > ' + str(z_score_threshold),
                   fontsize=16)
        plt.legend(fontsize=12)

    return network_overlap_size, random_network_overlap_sizes


def transform_edges(G, method='cosine_sim', edge_weight_threshold=0.95):
    """
    Transforms binary edges using selected method (currently only cosine similarity is implemented).
    Cosine similarity measures the similarity between neighbors of node pairs in the input network

    :param G: network whose edges will be transformed
    :type G: :py:class:`networkx.Graph`
    :param method: Method to use, only ``cosine_sim`` supported. Any other value will
                   cause this method to output a warning and immediately return
    :type method: str
    :param edge_weight_threshold: Transformed edges will be returned which have values greater than this
    :type edge_weight_threshold: float
    :return: Graph with nodes identical to input G, but with transformed edges (values > edge_weight_threshold)
    :rtype: :py:class:`networkx.Graph`
    """

    if method not in ['cosine_sim']:  # update this if we add more methods
        warnings.warn('Error: ' + method + ' method not yet implemented')
        return

    # compute the adjacency matrix
    logging.info('computing the adjacency matrix...')

    nodelist = list(G.nodes())
    # get graph as adjacency matrix
    graph_as_adj = nx.to_numpy_array(G)

    # add transpose to matrix to remove edge direction
    adj_temp = graph_as_adj + np.transpose(graph_as_adj)

    # compute the cosine similarity
    logging.info('computing the cosine similarity...')
    cos_pc = pd.DataFrame(np.zeros((adj_temp.shape[0],
                                   adj_temp.shape[1])),
                          index=nodelist)
    cos_pc.columns = nodelist

    for i in np.arange(0, len(nodelist)-1):
        n1 = nodelist[i]
        # this node has no neighbors so set
        # cosine distance to maximum distance aka 1.0
        # in cos_pc and continue
        if max(adj_temp[i]) < 1.0:
            for j in np.arange(i + 1, len(nodelist)):
                n2 = nodelist[j]
                cos_pc.loc[n1][n2] = 1.0
                cos_pc.loc[n2][n1] = 1.0
            continue
        for j in np.arange(i+1, len(nodelist)):
            n2 = nodelist[j]

            # make sure they have some neighbors
            if max(adj_temp[j]) > 0.0:
                cosine_distance = distance.cosine(adj_temp[i],
                                                  adj_temp[j])
            else:
                # no neighbors in neigh2 so set
                # cosine distance to maximum distance aka 1.0
                cosine_distance = 1.0

            cos_pc.loc[n1][n2] = cosine_distance
            cos_pc.loc[n2][n1] = cosine_distance

    # Rank transform 1-cos distance
    logger.info('rank transforming...')
    m1cos = 1-cos_pc
    m1cos = m1cos.replace(np.nan, 0)
    sim_names = m1cos.index.tolist()
    sim_rank = m1cos.rank(0) / (m1cos.shape[0] - 1)
    sim_rank = pd.DataFrame((sim_rank.values + sim_rank.values.T) / 2.0,
                            columns=sim_names, index=sim_names)

    # remove self edges
    sim_rank.values[[np.arange(sim_rank.shape[0])]*2] = 0

    sim_rank['gene_temp'] = sim_rank.index.tolist()
    sim_rank_EL = sim_rank.melt(id_vars=['gene_temp'])
    sim_rank_EL.columns = ['node1', 'node2', 'sim']
    sim_rank_EL = sim_rank_EL[sim_rank_EL['sim'] > edge_weight_threshold]
    G_transf = nx.Graph()
    G_transf.add_nodes_from(G)
    G_transf.add_weighted_edges_from(zip(sim_rank_EL['node1'],
                                         sim_rank_EL['node2'],
                                         sim_rank_EL['sim']))

    logger.info('number of transformed edges returned = ' +
                str(len(G_transf.edges())))

    return G_transf




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re
from setuptools import setup, find_packages

with open(os.path.join('netcoloc', '__init__.py')) as ver_file:
    for line in ver_file:
        if line.startswith('__version__'):
            version=re.sub("'", "", line[line.index("'"):])

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'click>=6.0',
    'ndex2',
    'networkx>=2.0',
    'mygene>=3.2.2',
    'scipy>=1.5.3',
    'numpy',
    'pandas',
    'tqdm',
    'matplotlib',
    'seaborn',
    'statsmodels',
    'gprofiler-official>=1.0.0'
]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name='netcoloc',
    version=version,
    description="",
    long_description=readme + '\n\n' + history,
    author="Brin Rosenthal, Sophie Liu",
    author_email='sbrosenthal@health.ucsd.edu, sol015@ucsd.edu',
    url='https://github.com/ucsd-ccbb/netcoloc',
    packages=find_packages(include=['netcoloc']),
    package_dir={'netcoloc':
                 'netcoloc'},
    entry_points={
        'console_scripts': [
            'netcoloc=netcoloc.cli:main'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords='netcoloc',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    setup_requires=['wheel']
)



# -*- coding: utf-8 -*-

import click

from .netprop_zscore import *

@click.command(context_settings={"ignore_unknown_options": True})
@click.argument('seed-gene-file', type=click.Path(exists=True, resolve_path=True))
@click.option('--seed-gene-file-delimiter')
@click.option('--num-reps', default=10, type=int)
@click.option('--alpha', '-a', default=0.5, type=float)
@click.option('--minimum-bin-size', default=10, type=int)
@click.option('--interactome-file')
@click.option('--interactome-uuid', default='f93f402c-86d4-11e7-a10d-0ac135e8bacf', type=click.UUID)
@click.option('--ndex-server', default='ndexbio.org')
@click.option('--ndex-user')
@click.option('--ndex-password')
@click.option('--out-name', default='out')
@click.option('--save-z-scores/--no-save-z-scores', default=True)
@click.option('--save-final-heat/--no-save-final-heat', default=False)
@click.option('--save-random-final-heats/--no-save-random-final-heats', default=False)
def main(seed_gene_file, seed_gene_file_delimiter, num_reps, alpha, minimum_bin_size, interactome_file,
                   interactome_uuid, ndex_server, ndex_user, ndex_password, out_name, save_z_scores, save_final_heat,
                   save_random_final_heats):
  netprop_zscore(
    seed_gene_file, seed_gene_file_delimiter=seed_gene_file_delimiter, num_reps=num_reps, alpha=alpha,
    minimum_bin_size=minimum_bin_size, interactome_file=interactome_file, interactome_uuid=str(interactome_uuid),
    ndex_server=ndex_server, ndex_user=ndex_user, ndex_password=ndex_password, out_name=out_name,
    save_z_scores=save_z_scores, save_final_heat=save_final_heat, save_random_final_heats=save_random_final_heats
  )

if __name__ == "__main__":
  main()
